--	Giddyup: a World of Warcraft addon to choose a random mount.
--	Copyright (C) 2014  Gurth

--	This program is free software; you can redistribute it and/or modify
--	it under the terms of the GNU General Public License as published by
--	the Free Software Foundation; either version 2 of the License, or
--	(at your option) any later version.

--	This program is distributed in the hope that it will be useful,
--	but WITHOUT ANY WARRANTY; without even the implied warranty of
--	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--	GNU General Public License for more details.

--	You should have received a copy of the GNU General Public License
--	along with this program; if not, write to the Free Software
--	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



local DISMOUNT = "dismount"
local MOUNT_ID = "id"
local MOUNT_NAME = "name"
local MOUNT_TYPE_ANY = "any"
local MOUNT_TYPE_FLYING = "fly"
local MOUNT_TYPE_GROUND = "ground"
local MOUNT_TYPE_PASSENGER = "pass"
local MOUNT_TYPE_REPAIR = "repair"
local MOUNT_TYPE_SWIMMING = "swim"


function GiddyUp(msg, editBox)

	local command, options = string.match(msg, "^(%S*)%s*(.-)$")

	if command == MOUNT_TYPE_ANY or
	   command == MOUNT_TYPE_FLYING or
	   command == MOUNT_TYPE_GROUND or
	   command == MOUNT_TYPE_SWIMMING then
		if IsMounted() then
			Giddyup_DismountMe()
		else
			Giddyup_MountMeByType(command, options)
		end
	elseif command == MOUNT_TYPE_PASSENGER or
		   command == MOUNT_TYPE_REPAIR then
		if IsMounted() then
			Giddyup_DismountMe()
		else
			Giddyup_MountMeByKind(command, options)
		end
	elseif string.sub(command, 1, 1) == "(" and string.sub(string.reverse(command), 1, 1) == ")" then
		if IsMounted() then
			Giddyup_DismountMe()
		else
			Giddyup_MountMeByPattern(command, options)
		end
	elseif command == DISMOUNT then
		Giddyup_DismountMe(options)
	else
		Giddyup_Help()
	end
end


function Giddyup_DismountMe(dismountSpeech)
	if IsFlying() then
		print("Giddyup: You're flying. Land before attempting to dismount if you care about your repair bill.")
	else
		if IsMounted() then
			for n = 0, C_MountJournal.GetNumMounts() do
				creatureName, _, _, active, _, _, _, _, _, _, _ = C_MountJournal.GetMountInfo(n)
				if active and dismountSpeech ~= nil then
					Giddyup_Speech(string.format(dismountSpeech, creatureName))
					break
				end
			end
		end
		C_MountJournal.Dismiss()
	end
end


function Giddyup_Help()
	print("Slash commands available for Giddyup:")
	print("  /giddyup - display this message")
	print(string.format("  /giddyup %s - random mount of any type", MOUNT_TYPE_ANY))
	print(string.format("  /giddyup %s - dismount from current mount", DISMOUNT))
	print(string.format("  /giddyup %s - random flying mount", MOUNT_TYPE_FLYING))
	print(string.format("  /giddyup %s - random ground mount", MOUNT_TYPE_GROUND))
	print(string.format("  /giddyup %s - random mount that can carry passenger(s)", MOUNT_TYPE_PASSENGER))
	print(string.format("  /giddyup %s - random mount that carries a repair NPC", MOUNT_TYPE_REPAIR))
	print(string.format("  /giddyup %s - random swimming mount", MOUNT_TYPE_SWIMMING))
	print("  /giddyup (pattern) - random mount whose name contains pattern (parentheses must be included)")
	print("  /giddyup (!pattern) OR /giddyup ('pattern) - random mount whose name does not contain pattern (parentheses must be included)")
	print("  /gup - alternative for /giddyup, can be used with all commands above")
	print("All except plain /giddyup (or /gup) can have a message added after the command, for character to /say when mounting. To include mount's name in message, use `%s'; to change channel, prefix message by channel name: /g, /o or /p.")
	print("(Tip: Scroll back up to read all of this help message.)")
end


function Giddyup_MountMeByKind(mountKindDesired, mountUpSpeech)

	local MOUNT_ID = "id"
	local MOUNT_NAME = "name"
	local mountsDesired = {}
	
	if mountKindDesired == MOUNT_TYPE_PASSENGER then
		mountsDesired = {
			[188] = true,	-- Grand Black War Mammoth (Horde)
			[189] = true,	-- Grand Black War Mammoth (Alliance)
			[192] = true,	-- Grand Expedition Yak
			[194] = true,	-- Grand Ice Mammoth (Horde)
			[195] = true,	-- Grand Ice Mammoth (Alliance)
			[270] = true,	-- Mechano-Hog
			[271] = true,	-- Mekgineer's Chopper
			[443] = true,	-- Traveler's Tundra Mammoth (Horde)
			[444] = true,	-- Traveler's Tundra Mammoth (Alliance)
			[496] = true	-- X-53 Touring Rocket
		}
	elseif mountKindDesired == MOUNT_TYPE_REPAIR then
		mountsDesired = {
			[192] = true,	-- Grand Expedition Yak
			[443] = true,	-- Traveler's Tundra Mammoth (Horde)
			[444] = true	-- Traveler's Tundra Mammoth (Alliance)
		}
	end
	
	-- Find all mounts player actually has of desired kind
	local mountsCollected = {}
	for mountID = 0, C_MountJournal.GetNumMounts() do
		local creatureName, _, _, _, isUsable, _, _, _, _, _, _ = C_MountJournal.GetMountInfo(mountID)
		if isUsable and mountsDesired[mountID] ~= nil then
			mountsCollected[#mountsCollected+1] = {[MOUNT_NAME] = creatureName, [MOUNT_ID] = mountID}
		end
	end	
	
	-- And now choose a suitable one
	GiddyUp_SummonRandomMount(mountsCollected, mountUpSpeech, mountKindDesired)
end


function Giddyup_MountMeByPattern(pattern, mountUpSpeech)

	-- Chop off the parentheses around the pattern
	pattern = string.lower(string.sub(pattern, 2, #pattern - 1))
	
	-- Check if we're dealing with a "negate" pattern
	local negatePattern = false
	if string.sub(pattern, 1, 1) == "!" or string.sub(pattern, 1, 1) == "'" then
		negatePattern = true
		pattern = string.sub(pattern, 2)
	end
	
	local mountTypes = {}

	-- Find all mounts player actually has of desired type
	local mountsCollected = {}
	for mountID = 0, C_MountJournal.GetNumMounts() do
		local creatureName, _, _, _, isUsable, _, _, _, _, _, _ = C_MountJournal.GetMountInfo(mountID)
		if negatePattern then
			-- Find mounts whose names _don't_ contain pattern
			if isUsable and string.find(string.lower(creatureName), pattern) == nil then
				mountsCollected[#mountsCollected+1] = {[MOUNT_NAME] = creatureName, [MOUNT_ID] = mountID}
			end
		else
			-- Find mounts whose names contain pattern
			if isUsable and string.find(string.lower(creatureName), pattern) ~= nil then
				mountsCollected[#mountsCollected+1] = {[MOUNT_NAME] = creatureName, [MOUNT_ID] = mountID}
			end
		end
	end	
	
	-- And now choose a suitable one
	GiddyUp_SummonRandomMount(mountsCollected, mountUpSpeech, pattern)
end


function Giddyup_MountMeByType(mountTypeDesired, mountUpSpeech)

	local mountTypes = {}
	
	if mountTypeDesired == MOUNT_TYPE_ANY then
		for n = 230, 269 do
			mountTypes[n] = MOUNT_TYPE_ANY
		end
	else
		mountTypes[230] = MOUNT_TYPE_GROUND
		mountTypes[231] = MOUNT_TYPE_SWIMMING	-- Riding Turtle and Sea Turtle, only the latter actually being a swimming mount, but Riding Turtle is pretyt rare so owners will just have to deal with it being summoned occasionally
		mountTypes[232] = MOUNT_TYPE_SWIMMING
		mountTypes[241] = MOUNT_TYPE_GROUND
		mountTypes[247] = MOUNT_TYPE_FLYING
		mountTypes[248] = MOUNT_TYPE_FLYING
		
		-- Make Giddyup pick water striders etc. if swimming
		if IsSwimming() then
			mountTypes[269] = MOUNT_TYPE_SWIMMING
		else
			mountTypes[269] = MOUNT_TYPE_GROUND
		end
	end
	
	-- Find all mounts player actually has of desired type
	local mountsCollected = {}
	for mountID = 0, C_MountJournal.GetNumMounts() do
		local creatureName, _, _, _, isUsable, _, _, _, _, _, _ = C_MountJournal.GetMountInfo(mountID)
		local _, _, _, _, mountType = C_MountJournal.GetMountInfoExtra(mountID)
		if isUsable and mountTypes[mountType] == mountTypeDesired then
			mountsCollected[#mountsCollected+1] = {[MOUNT_NAME] = creatureName, [MOUNT_ID] = mountID}
		end
	end	
	
	-- And now choose a suitable one
	GiddyUp_SummonRandomMount(mountsCollected, mountUpSpeech, mountTypeDesired)
end


function Giddyup_Speech(text)

	local CHAT_GUILD = "GUILD"
	local CHAT_OFFICER = "OFFICER"
	local CHAT_PARTY = "PARTY"
	local CHAT_SAY = "SAY"

	if text ~= nil then
		local channel = CHAT_SAY
		-- Check if user specified chat channel
		-- Giddyup only supports a few, to prevent spamming or reaching too wide an audience
		if string.sub(text, 1, 1) == "/" then
			local tempChannel, tempText = string.match(text, "^(%S*)%s*(.-)$")
			text = tempText
			if tempChannel == "/g" or tempChannel == "/guild" then
				channel = CHAT_GUILD
			elseif tempChannel == "/p" or tempChannel == "/party" then
				channel = CHAT_PARTY
			elseif tempChannel == "/o" or tempChannel == "/officer" then
				channel = CHAT_OFFICER
			end
		end
		SendChatMessage(text, channel)
	end
end


function GiddyUp_SummonRandomMount(mountsAvailable, mountUpSpeech, mountTypeDesired)
	if #mountsAvailable == 0 then
		print(string.format("Giddyup: You don't seem to have any \"%s\" mounts.", mountTypeDesired))
	else
		local chosenMount = mountsAvailable[random(1, #mountsAvailable)]
		Giddyup_Speech(string.format(mountUpSpeech, chosenMount[MOUNT_NAME]))
		C_MountJournal.Summon(chosenMount[MOUNT_ID])
	end
end


SLASH_GIDDYUP1, SLASH_GIDDYUP2 = "/giddyup", "/gup";
SlashCmdList["GIDDYUP"] = GiddyUp